/*
 *	IL interface manager implementation
 *	issued: 2000/4/5
 *	author: c.mos@zob.ne.jp
 */

#include "std.h"
#include "ilib.h"
#include "remote.h"

/* static work */

typedef struct {
	WORD magic1, magic2;
	BYTE fill[60];
	fent_t fent;
} FBFHeader;

FBFHeader header;

static FS dir;
static fmode_t fmask;
static FSID cur_fsid;

FS get_fs(FSID fsid)
{
	FS fs;

	cur_fsid = fsid;
	switch(fsid) {
		case FS_FLASH: return rom0_fs;
		case FS_SRAM:  return ram0_fs;
	}
	return NULL;
}

void set_current_fs(FSID fsid)
{
	dir = get_fs(fsid);
}

void set_mask(fmode_t _mask)
{
	fmask = _mask;
}

#undef cwfs
#define cwfs dir

static BOOL get_entry1(int index, fent_t __ss *fentp)
{
	return	get_il(dir)->_getent(dir, index, fentp) == E_FS_SUCCESS && fentp->count != -1 &&  !(fentp->mode & fmask);
}

int get_file_count()
{
	fent_t fent;
	int files = 0;
	int i;

	for_to(i, dir->count) {
		if(get_entry1(i, &fent)) files++;
	}
	return files;
}

BOOL get_entry_tmp(int index, fent_t __ss *fentp)
{
	int count = 0;
	int i;

	for_to(i, dir->count) {
		if(get_entry1(i, fentp) && count++ == index) return TRUE;
	}
	return FALSE;
}

fent_t *get_entry(int index)
{
	fent_t fent;
	int count = 0;
	int i;

	if(get_entry_tmp(index, &fent)) {
		memcpy(&header.fent, &fent, sizeof(fent_t));
		return &header.fent;
	}
	return NULL;
}

int select_file(char far *name)
{
	fent_t fent;
	int index = 0;
	int i;

	for_to(i, dir->count) {
		if(get_entry1(i, &fent)) {
			if(strncmp(fent.name, name, MAXFNAME) == 0) return index;
			index++;
		}
	}
	return 0;
}

void get_fullpath(char far *buffer)
{
	*buffer++ = '/';
	buffer += lstrncpy(buffer, dir->name, MAXPATHLEN);
	*buffer++ = '/';
	buffer += lstrncpy(buffer, header.fent.name, MAXPATHLEN);
	*buffer = '\0';
}

ulong get_free_space()
{
	return get_il(dir)->_space(dir);
}

void far *get_location(floc_t loc)
{
	loc.w.seg = (loc.w.seg << 12) + 0x8000;
    return MK_FP(far2seg(loc.ul), 0);
}

void exec_file(char *app)
{
	void far *entry;
	char far *argv[2];
	char file_path[MAXPATHLEN];
	char app_path[MAXFNAME*2];
	char far *command = file_path;
	int argc = 1;

	get_fullpath(file_path);
	if(app) {
		strcpy(app_path, "/rom0/");
		strcat(app_path, app);
		command = app_path;
		argc++;
	} 
	if((entry = procIL->_load(command))) {
		argv[0] = command;
		if(app) 
			argv[1] = file_path;
		if(remote.mode) remote_disable();
		procIL->_run(entry, argc, argv);
		if(remote.mode) remote_enable();
	}
}

void delete_file()
{
	char path[MAXPATHLEN];
	get_fullpath(path);
	get_il(dir)->_unlink(dir, path);
}

void call_reboot(int (far *fp)(FS fs))
{
	if(cur_fsid == FS_FLASH) {
		asm push word ptr dir+2
		asm push word ptr dir
		asm push 0xFFFF
		asm push 0
		asm jmp dword ptr fp
	}
	fp(dir);
}

void new_fs()
{
	call_reboot(get_il(dir)->_newfs);
}

void deflag()
{
	call_reboot(get_il(dir)->_defrag);
/*	get_il(dir)->_defrag(dir);*/
}

/*
 * xmodem
 */

#define MAGIC1 '#!'
#define MAGIC2 'ws'

static xmodeminfo xminfo;

int receive_file(int (*callback)(xmodeminfo *, fent_t *))
{
	fent_t *fs = NULL;

	xminfo.state = XM_START;
	xminfo.mode	 = XMODE_RECEIVE;
	xminfo.block_max = 512;
	xminfo.bank = BANK_OSWORK;
	xminfo.offset = (int)&header + ((_DS - 0x1000) << 4);

	do {
		floc_t location;
		result_t result;
		comm_xmodem(&xminfo);
		if(!fs && xminfo.state == XM_BLOCK) {
			if(header.magic1 != MAGIC1 || header.magic2 != MAGIC2)
				return E_ILIB_NOT_FBF;
			fs = &header.fent;
			get_il(dir)->_unlink(dir, fs->name);
			location.ul = dir->loc.ul + dir->resource; /*free area start offset*/
			xminfo.bank	  = location.w.seg;
			xminfo.offset = location.w.off;
			result = get_il(dir)->_creat(dir, fs);
			if(result != E_FS_SUCCESS) return result;
		}
		if(callback && callback(&xminfo, fs) < 0) break;
		if(xminfo.state == XM_DONE) return 0;
	} while(xminfo.state > 0);
	if(fs)
		get_il(dir)->_unlink(dir, fs->name);
	return xminfo.state;
}

int send_file(int (*callback)(xmodeminfo *, fent_t *))
{
	floc_t location;
	header.magic1 = MAGIC1;
	header.magic2 = MAGIC2;
	memset(&header.fill, 0xFF, sizeof(header.fill));
	location = header.fent.loc;
	header.fent.loc.ul = 0;

	xminfo.state = XM_START;
	xminfo.mode	 = XMODE_SEND;
	xminfo.block_max = header.fent.count + 1;
	xminfo.bank = BANK_OSWORK;
	xminfo.offset = (int)&header + ((_DS - 0x1000) << 4);

	do {
		result_t result;
		comm_xmodem(&xminfo);
		if(xminfo.block_c == 1 && xminfo.state == XM_BLOCK) {
			xminfo.bank	  = location.w.seg;
			xminfo.offset = location.w.off;
		}
		if(callback && callback(&xminfo, &header.fent) < 0) break;
		if(xminfo.state == XM_DONE) return 0;
	} while(xminfo.state > 0);
	return xminfo.state;
}

/* extern WORD _os_ds; */

size_t read_file(FSID fsid, char far *name, void *buffer, size_t size)
{
	int fd;
	FS fs;
	fs = get_fs(fsid);
	fd = get_il(fs)->_open(fs, name, FMODE_R, 0);
	if(fd == -1) return 0;
	size = get_il(fs)->_read(fd, buffer, size);
	get_il(fs)->_close(fd);
	return size;
}

size_t write_file(FSID fsid, char far *name, void *buffer, size_t size)
{
	int fd;
	FS fs = get_fs(fsid);
	fd = get_il(fs)->_open(fs, name, FMODE_W, 0);
	if(fd == -1) return 0;
	size = get_il(fs)->_write(fd, buffer, size);
	get_il(fs)->_close(fd);
	return size;
}

unsigned os_get_version()
{
	return sramwork_p->_os_version;
}

size_t read_resource_header(FSID fsid, FS fs, res_t __ss *res)
{
	int fd;
	size_t size;
	FS dir = get_fs(fsid);
	fd = get_il(dir)->_open(dir, fs->name, FMODE_R, 0);
	if(fd == -1) return 0;
	get_il(dir)->_lseek(fd, fs->resource, 0);
	size = get_il(dir)->_read(fd, res, sizeof(res_t));
	get_il(dir)->_close(fd);
	return size;
}
