/*
 *	Menu manager implementation
 *	issued: 2000/4/3
 *	author: c.mos@zob.ne.jp
 */

/* #define BMPSAVE */

#ifdef BMPSAVE
#include "../bmpsaver.h"
extern int sprite_used;
#endif

#include "menu.h"

#define CHECKMARK 0x14
#define RADIOMARK 0x15

static Rect rect_bar = { 0, 0, LCD_CHAR_CX, 1 };

static int get_item_color(MenuItem* item, BOOL sel)
{
	return (item->flags & MF_GRAYED) ? (sel ? CLR_GRAY_HIGHLIGHT : CLR_GRAY)
									 : (sel ? CLR_HIGHLIGHT : CLR_NORMAL);
}

static BOOL is_item_valid(MenuItem* item)
{
	return !(item->flags & MF_GRAYED) && item->label;
}

void init_menu_position(MenuBar* bar)
{
	int i, j;
	int x;
	x = rect_bar.x + 1;
	for_to(i, bar->menu_count) {
		MenuItem* item = &bar->items[i];
		Menu* menu = &bar->menus[i];
		int cx_max = 0;
		for_to(j, menu->item_count) {
			MenuItem* item = &menu->items[j];
			int cx = string_width(item->label, 0) + 1;
			if(cx > cx_max) cx_max = cx;
		}
		menu->wnd.cx = cx_max;
		menu->wnd.cy = menu->item_count;
		menu->wnd.x = x;
		if(x + menu->wnd.cx > LCD_CHAR_CX - 2)
			menu->wnd.x = LCD_CHAR_CX - 1 - menu->wnd.cx;
		menu->wnd.y = rect_bar.y + 2;
		x += string_width(item->label, 0) + 1;
	}
}

void draw_menubar(MenuBar* bar)
{
	int i;
	int x = rect_bar.x + 1;
	draw_separator(rect_bar.x, rect_bar.y + 1, rect_bar.cx, FRAME_BTM, PAL_FRAME);
	rect_bar.cy = 1;
	fill_rect(&rect_bar, PAL_WINDOW, CLR_NORMAL);
	for_to(i, bar->menu_count) {
		MenuItem* item = &bar->items[i];
		char* title = item->label;
		set_color(get_item_color(item, (bar->menu_sel == i)));
		x += text_put_string(x++, rect_bar.y, title);
	}
}

void erase_menubar()
{
	rect_bar.cy = 2;
	fill_rect(&rect_bar, PAL_DESKTOP, CLR_DESKTOP);
}

void draw_menu(Menu* menu)
{
	int i;
	draw_frame(&menu->wnd);
	set_palette_rect(&menu->wnd, PAL_WINDOW);
	for_to(i, menu->item_count) {
		draw_menuitem(menu, i);
	}
}

void erase_menu(Menu* menu)
{
	erase_frame(&menu->wnd);
	draw_desktop(&menu->wnd);
}

void draw_menuitem(Menu* menu, int i)
{
	int x = menu->wnd.x;
	int y = menu->wnd.y + i;
	int fillchar = ' ';
	MenuItem* item = &menu->items[i];
	if(item->label) {
		int col = get_item_color(item, (menu->item_sel == i));
		int check = (item->flags & MF_RADIOCHECK) ? RADIOMARK : (item->flags & MF_CHECKED) ? CHECKMARK : ' ';
		set_color(col);
		text_put_char(x++, y, check);
		x += text_put_string(x, y, item->label);
		text_fill_char(x, y, menu->wnd.x + menu->wnd.cx - x, fillchar);
	} else {
		draw_separator(x, y, menu->wnd.cx, FRAME_SEP, PAL_BTNFRAME);
/*		set_color(CLR_NORMAL);
		fillchar = 0x849F;	* '' */
	}
}

void draw_menu_help(Menu* menu, int i)
{
	int x = 0;
	MenuItem* item = &menu->items[i];
	set_color(CLR_STATUS);
	if(item->help)
		x = text_put_string(0, STATUS_Y, item->help);
	text_fill_char(x, STATUS_Y, LCD_CHAR_CX - x, ' ');
}

int do_menu(MenuBar* bar)
{
	int item0;
	Menu* menu;
	int id;
	init_menu_position(bar);
Redraw:
	menu = &bar->menus[bar->menu_sel];
	menu->item_sel = 0;
	if(menu->items[0].flags & MF_RADIOCHECK) menu->item_sel++;
	draw_menubar(bar);
	draw_menu(menu);
	for(;;) {
		int i;
		item0 = menu->item_sel;
		draw_menu_help(menu, item0);
		switch(key_wait()) {
#ifdef BMPSAVE
		case KEY_START:
			BREAK;
			bs_sprite_set_range(0, sprite_used);
			bs_save_bmp("lcd.bmp", 0, 0, 224, 144);
			break;
#endif			
		case KEY_DOWN1:
			for_to(i, menu->item_count) {
				if(++menu->item_sel >= menu->item_count) menu->item_sel = 0;
				if(menu->items[menu->item_sel].label) break;
			}
			goto UpDown;
		case KEY_UP1:
			for_to(i, menu->item_count) {
				if(--menu->item_sel < 0) menu->item_sel = menu->item_count - 1;
				if(menu->items[menu->item_sel].label) break;
			}
UpDown:		draw_menuitem(menu, item0);
			draw_menuitem(menu, menu->item_sel);
			continue;
		case KEY_RIGHT1:
			for_to(i, bar->menu_count) {
				if(++bar->menu_sel >= bar->menu_count) bar->menu_sel = 0;
				if(is_item_valid(&bar->items[bar->menu_sel])) break;
			}
			goto LeftRight;
		case KEY_LEFT1:
			for_to(i, bar->menu_count) {
				if(--bar->menu_sel < 0) bar->menu_sel = bar->menu_count - 1;
				if(is_item_valid(&bar->items[bar->menu_sel])) break;
			}
LeftRight:	erase_menu(menu);
			goto Redraw;
		case KEY_A: {
			MenuItem* item = &menu->items[menu->item_sel];
			if(!(item->flags & MF_GRAYED)) {
				id = menu->items[menu->item_sel].id;
				goto EndMenu;
			}
			break;
		}
		case KEY_B:
			id = -1;
			goto EndMenu;
		}
	}
EndMenu:
	erase_menu(menu);
	erase_menubar();
	return id;
}

MenuItem* search_menuitem(MenuBar* bar, int id)
{
	int i, j;
	for_to(i, bar->menu_count) {
		Menu* menu = &bar->menus[i];
		for_to(j, menu->item_count) {
			MenuItem* item =  &menu->items[j];
			if(item->id == id) return item;
		}
	}
	return NULL;
}

void menu_set_enable(MenuBar* bar, int id, BOOL enable)
{
	MenuItem* item = search_menuitem(bar, id);
	if(item) {
		if(enable)
			item->flags &= ~MF_GRAYED;
		else
			item->flags |= MF_GRAYED;
	}
}

void menu_set_check(MenuBar* bar, int id, BOOL check)
{
	MenuItem* item = search_menuitem(bar, id);
	if(item) {
		if(check)
			item->flags |= MF_CHECKED;
		else
			item->flags &= ~MF_CHECKED;
	}
}

void menu_set_radio(MenuBar* bar, int id, int count, int select)
{
	MenuItem* item = search_menuitem(bar, id);
	if(item) {
		int i;
		for_to(i, count) {
			if(i == select)
				item->flags |= MF_RADIOCHECK;
			else
				item->flags &= ~MF_RADIOCHECK;
			item++;
		}
	}
}

