/*
 *  profile manager implementation
 *	issued: 2000/4/11
 *	author: c.mos@zob.ne.jp
 */

#include "std.h"
#include "ilib.h"
#include "prof.h"

#define PROF_MAX 1024
#define EOF 0x1A

#define isspace(c) ((BYTE)c <= ' ')

typedef struct {
	BOOL read;
	char *name;
	char *buffer;
	char *sectp;
	char *p;
	size_t len;
} Profile;

static Profile prof;

extern void *_heap;

BOOL open_profile(BOOL read, char* name, int size)
{
	prof.read = read;
	prof.name = name;
	prof.buffer = _heap;
	prof.p = prof.sectp = prof.buffer;
	if(read) {
		prof.len = read_file(FS_SRAM, prof.name, prof.buffer, PROF_MAX);
		*(prof.p + prof.len) = EOF;
		return prof.len != 0;
	} else { /* write */
		prof.len = 0;
		return TRUE;
	}
}

BOOL close_profile()
{
	if(prof.read) return TRUE;
	prof.len = prof.p - prof.buffer;
	return write_file(FS_SRAM, prof.name, prof.buffer, prof.len) != 0;
}

enum { READ_KEY, READ_SEP, READ_VALUE };

/*
ProfType read_profile(char* keybuff, char* valuebuff)
{
	int c;
	int keylen = 0;
	int mode = READ_KEY;

	while((c = *prof.p++) != EOF) {
		if(mode == READ_KEY) {
			if(c == '[') continue;
			if(c == ']') {
				*keybuff = *valuebuff = '\0';
				return PROF_SECTION;
			}
			if(c == '=') {
				*keybuff = '\0';
				mode = READ_SEP;
			} else if(!isspace(c)) {
				*keybuff++ = (char)c;
				keylen++;
			}
		} else if(mode == READ_SEP) {
			if(!isspace(c)) mode = READ_VALUE;
		}
		if(mode == READ_VALUE) {
			if(c == '\r' || c == '\n') {
				*valuebuff = '\0';
				return PROF_KEY;
			}
			*valuebuff++ = (char)c;
		}
	}
	return PROF_EOF;
}
*/

char *skipline(char *p)
{
	char c;
	while((c = *p) != EOF) {
		p++;
		if(c == '\n') break;
	}
	return p;
}
void copyline(char* buff, char* p)
{
	while((BYTE)*p >= ' ') *buff++ = *p++;
	*buff = '\0';
}

int getnum(char * __ss *pp)
{
	char c;
	int n = 0;
	char *p = *pp;
	while((c = *p) >= '0' && c <= '9') {
		n = n * 10 + c - '0';
		p++;
	}
	*pp = p;
	return n;
}

BOOL read_profile_section(char *sectp)
{
	BOOL found = FALSE;
	char *p = prof.buffer;
	char c;
	int sectlen = strlen(sectp);
	while((c = *p++) != EOF) {
		if(isspace(c)) continue;
		if(c == '[') {
			if(strncmp(sectp, p, sectlen) == 0)
				found = TRUE;
		}
		p = skipline(p);
		if(found) {
			prof.p = prof.sectp = p;
			return TRUE;
		}
	}
	return FALSE;
}

char *scan_profile_key(char *key)
{
	char *p = prof.sectp;
	char c;
	int keylen = strlen(key);
	while((c = *p++) != EOF) {
		if(isspace(c)) continue;
		if(c == '[') return NULL;
		if(strncmp(key, p-1, keylen) == 0)
			return p + keylen;
		p = skipline(p);
	}
	return NULL;
}

BOOL read_profile_next_key(char *keybuff)
{
	char c;
	while((c = *prof.p++) != EOF) {
		if(isspace(c)) continue;
		if(c == '[') return FALSE;
		prof.p--;
		while(*prof.p != '=') *keybuff++ = *prof.p++;
		*keybuff = '\0';
		prof.p = skipline(prof.p);
		return TRUE;
	}
	return FALSE;
}

void read_profile_string(char *key, char *s_default, char *sbuff)
{
	char *p = scan_profile_key(key);
	if(p)
		copyline(sbuff, p);
	else if(s_default)
		strcpy(sbuff, s_default);
}

int read_profile_int(char *key, int n_default)
{
	char *p = scan_profile_key(key);
	return p ? getnum(&p) : n_default;
}

int read_profile_intarray(char *key, int* arr, int count)
{
	int i = 0;
	char *p = scan_profile_key(key);
	if(p) {
		for_to(i, count) {
			*arr++ = getnum(&p);
			if(*p++ != ',') break;
		}
	}
	return i;
}

void write_profile_section(char *s)
{
	prof.p += sprintf(prof.p, "[%s]\r\n", s);
}

void write_profile_string(char *key, char *s)
{
	prof.p += sprintf(prof.p, "%s=%s\r\n", key, s);
}

void write_profile_int(char *key, int n)
{
	prof.p += sprintf(prof.p, "%s=%d\r\n", key, n);
}

void write_profile_intarray(char *key, int* arr, int count)
{
	int i;
	prof.p += sprintf(prof.p, "%s=", key);
	for_to(i, count) {
		prof.p += sprintf(prof.p, "%d", *arr++);
		if(i < count - 1)
			*prof.p++ = ',';
	}
	*prof.p++ = '\r';
	*prof.p++ = '\n';
}
